# Search for occurences of links that point to other threads
# in this bboard and replace them with their new form.
#
# @author Tilmann Singer (tils@tils.net)

# The regular expression that matches the old link. Customize this to
# your needs. Must match the full link, and the old msg_id is expected
# to be in the first parenthesis submatch.
set old_link_regex {http://openacs.org/bboard/q-and-a-fetch-msg\.tcl\?msg_id=([0-9A-Za-z]+)(&topic(_id)?=[0-9A-Za-z%]*){0,2}}

# Full url and mount point of the new bboard package, without trailing
# slash.
set new_bboard_url "http://openacs.org/forums"


# Some notes:

# To get some testing material I wget'd a bunch of pages from
# openacs.org that appear as result to a search on "this
# thread". topic_id and topic are being used sometimes both, sometimes
# not at all, and sometimes they even contain each others value. It's
# important that they are fully matched by the regex. Hopefully the
# thing above does.

# What it would not match would be something like
# href="...&topic=OpenACS Testing" because the space would need some
# alternative regex. Seems that this never occurs anyway.

# There might be links to q-and-a-search that would be nice to replace
# too.

# Did not find any relative links in the wget'd stuff so they are not
# yet considered here. If they are added then I think care should be
# taken so that they are only replaced when inside a <a href...></a>
# clause to minimize the risk of confusing discussion about the bboard
# code itself.


ns_write "HTTP/1.0 200 OK
Content-type: text/plain

"

ns_write $old_link_regex

ns_write "

Looping through all messages to substitute links.

"

set test_counter 0

db_foreach get_message {
    select message_id, content from forums_messages order by message_id
} {
    ns_write "-- checking $message_id\n"

    # split content into a list where each element is either a
    # matching link or a text blurb in between.
    set content_list [list]
    set to_search $content
    while { [regexp -indices $old_link_regex $to_search match] } {
	lappend content_list [string range $to_search 0 [expr [lindex $match 0] - 1]]
	lappend content_list [string range $to_search [lindex $match 0] [lindex $match 1]]
	set to_search [string range $to_search [expr [lindex $match 1] +1] end]
    }
            
    if { [llength $content_list] == 0 } {
	# nothing to replace in this message
	continue
    }

    # append the tail
    lappend content_list $to_search
        
    set replaced_p 0

    for { set idx 0 } { $idx < [llength $content_list] } { incr idx } {
	set element [lindex $content_list $idx]

	if { [regexp $old_link_regex $element match old_id] } {
	    # this is a link

	    if { [db_0or1row get_ids_for_new_link { select new_message_id
		from migr_message_map
		where old_message_id=:old_id
	    }] } {
		# we found a mapping
		#set new_link "$new_bboard_url/message?message_id=$new_message_id&forum_id=$forum_id"
		set new_link "$new_bboard_url/message-view?message_id=$new_message_id"

		set content_list [lreplace $content_list $idx $idx $new_link]

		ns_write "test_counter: $test_counter
new message_id: $new_message_id
old_id: $old_id
new link: $new_link
"
		set replaced_p 1
		
	    } else {
		# no mapping found
		ns_write "warning: no mapping entry in migr_message_map for old id: $old_id\n"
	    }
	    
	    
	    incr test_counter
	    if { $test_counter > 100 } {
		ad_script_abort
	    }

	}
	
	if { $replaced_p } {
	    set new_content [join $content_list ""]
	    #db_dml update_content "update cr_revisions set content=:new_content where revision_id=:revision_id"
	    db_dml update_content "update forums_messages set content=:new_content where message_id=:message_id"
	}
    }
}

ns_write "


Done.
"
