
set olddb [ns_db gethandle acs32]

# NOTE: only migrates minimum: email, name, passwd, url, registration_date
#  for posting users

# added where and order by clauses (olah)
set selection [ns_db select $olddb "
  select last_name, first_names, email, password, url,
      user_id as old_user_id, registration_date,
      registration_ip, user_state
  from users
  order by user_id
"]

set n_tried_old 0

set new_user_sql "
select user_id as new_lookup_id, lower(email) from cc_users where lower(email)=lower(:email)"

set mapped_user_sql   "
select old_user_id as old_map_id,
new_user_id as new_map_id 
from migr_user_map where old_user_id=:old_user_id"


set nil [set new [set map [set update [list]]]]
set output ""
db_transaction {
while { [ns_db getrow $olddb $selection] } {
    set_variables_after_query

    # we'are mapping the user_state/member_state of the users thusly:
    # 3.x          <-->          4.5
    # need_email_verification_and_admin_approv  <-->  needs approval
    # need_admin_approv  <-->  needs approval
    # need_email_verification   <-->  needs approval
    # rejected  <-->  rejected
    # authorized  <-->  approved
    # deleted  <-->  deleted

    switch [string range $user_state 0 3] {
	auth { set member_state "approved" }
	dele { set member_state "deleted" }
	need { set member_state "needs approval" }
	reje { set member_state "rejected" }
	default { set member_state "rejected" }
    }

    incr n_tried_old
    if [db_0or1row tt $new_user_sql] {
	# a user with this email address is already on the new system.
	# if the accompanying user_id is not already in the mapping table, we insert a row.  
	# (we check this because someone may register with the new site
	# before we even run this script and will thus not exist in the mapping table.)

	if [db_0or1row r $mapped_user_sql] {
	    # this user_id has been mapped before, i.e, has been migrated.
	    set new_id $new_map_id
	    set action "nil"
        } else {
	    # user has registered with the new site and is therefore not yet
	    # in the mapping table.
	    db_dml q "insert into migr_user_map 
	    (old_user_id, new_user_id) values ($old_user_id, $new_lookup_id)"
	    set new_id $new_lookup_id
	    set action "map"
        }

    } elseif {[empty_string_p $first_names] || [empty_string_p $last_name] || [empty_string_p $email]} {
	# we safeguard against empty columns in the users table.
	set action "nil"

    } elseif {[db_0or1row tr $mapped_user_sql]} {
	# this email address is not on the new system but the user_id has been
	# mapped before. the user probably changed email address since the last
	# pass of this script... let's create a new user (or should we update the email address).

	set new_created_id [ad_user_new $email $first_names $last_name $password "" "" $url "t" $member_state "" $registration_date $registration_ip]
	set new_id $new_map_id
	set action "update"

    } else {
	# this email address is not on the new system, nor has the user_id been
	# mapped before. let's create this previously unseen user and register
	# the event in the mapping table.

	set new_created_id [ad_user_new $email $first_names $last_name $password "" "" $url "t" $member_state "" $registration_date $registration_ip]
	db_dml q "insert into migr_user_map 
	(old_user_id, new_user_id) values ($old_user_id, $new_created_id)"
	set new_id $new_created_id
	set action "new"
    }

    lappend $action "
    $old_user_id => $new_id:  
    $last_name, $first_names $email<br>"
    
    append output "<b>$action</b>:  $old_user_id => $new_id:  
    $last_name, $first_names $email<br>"
}
} on_error {
    set output "<h3>Error:</h3><pre>$errmsg</pre><br>
    <br>$output"
}

ns_db releasehandle $olddb
    
ns_return 200 text/html "<html>
<head><title>Migratonicuserfama</title></head>
<body><h2> Migrating Users</h2>
<table border='0' width='100%'>
<tr valign='top'>
  <td width='50%'>
    <h3>Unsorted Output:</h3>
    <h3>$n_tried_old actions</h3>
    $output
  </td>
  <td>
    <h3>Sorted Output:</h3>
    <h3>[llength $nil] nil -- users already created & mapped</h3>[join $nil]
    <h3>[llength $map] map -- users already created, but unmapped</h3>[join $map]
    <h3>[llength $update] update -- uncreated, but mapped</h3>[join $update]
    <h3>[llength $new] new -- uncreated and unmapped</h3>[join $new]
  </td>
</tr>
</table>
</body></html>
"


    
    
